@@ -2,14 +2,21 @@ package net.sf.jabref.oo;
 
 import com.sun.star.awt.Point;
 import com.sun.star.awt.XWindow;
+import com.sun.star.beans.XPropertyContainer;
 import com.sun.star.beans.XPropertySet;
 import com.sun.star.beans.PropertyValue;
+import com.sun.star.beans.XPropertySetInfo;
 import com.sun.star.comp.helper.Bootstrap;
 import com.sun.star.container.*;
+import com.sun.star.container.NoSuchElementException;
+import com.sun.star.document.XDocumentProperties;
+import com.sun.star.document.XDocumentPropertiesSupplier;
 import com.sun.star.frame.*;
 import com.sun.star.lang.*;
 import com.sun.star.lang.Locale;
 import com.sun.star.text.*;
+import com.sun.star.uno.Any;
+import com.sun.star.uno.Type;
 import com.sun.star.uno.UnoRuntime;
 import com.sun.star.uno.XComponentContext;
 import net.sf.jabref.*;
@@ -34,7 +41,7 @@ public class OOBibBase {
     final static String BIB_SECTION_NAME = "JR_bib";
     final static String BIB_SECTION_END_NAME = "JR_bib_end";
     final static String BIB_CITATION = "JR_cite";
-    final Pattern citePattern = Pattern.compile(BIB_CITATION+"\\d*_(\\d*)_(.*)");
+    public final Pattern citePattern = Pattern.compile(BIB_CITATION+"\\d*_(\\d*)_(.*)");
 
     final static int
         AUTHORYEAR_PAR = 1,
@@ -55,6 +62,9 @@ public class OOBibBase {
     XTextViewCursorSupplier xViewCursorSupplier = null;
     XComponent xCurrentComponent = null;
     XComponentLoader xComponentLoader = null;
+    XPropertyContainer userProperties = null;
+    XPropertySet propertySet = null;
+
     private boolean atEnd;
     private AlphanumericComparator entryComparator = new AlphanumericComparator();
     private YearComparator yearComparator = new YearComparator();
@@ -128,6 +138,12 @@ public class OOBibBase {
         // Access the text document's multi service factory:
         mxDocFactory = (XMultiServiceFactory) UnoRuntime.queryInterface(XMultiServiceFactory.class, mxDoc);
 
+
+        XDocumentPropertiesSupplier supp =
+                UnoRuntime.queryInterface(XDocumentPropertiesSupplier.class, mxDoc);
+        userProperties = supp.getDocumentProperties().getUserDefinedProperties();
+        propertySet = UnoRuntime.queryInterface(XPropertySet.class, userProperties);
+
     }
 
     public XDesktop simpleBootstrap(String pathToExecutable) throws Exception {
@@ -190,6 +206,34 @@ public class OOBibBase {
          return res;
      }
 
+    public void testCustomProperties() throws Exception {
+        XDocumentPropertiesSupplier supp = (XDocumentPropertiesSupplier)UnoRuntime.queryInterface(
+                XDocumentPropertiesSupplier.class, mxDoc);
+        XPropertyContainer cont = supp.getDocumentProperties().getUserDefinedProperties();
+        XPropertySet set = (XPropertySet)UnoRuntime.queryInterface(XPropertySet.class, cont);
+        try {
+            cont.addProperty("JR_cite_1_Danielsen1987", (short)0, new Any(Type.STRING, "Brum"));
+        } catch (Exception ex) {
+            System.out.println("property already exists");
+        }
+        System.out.println(set.getPropertyValue("Test").toString());
+
+    }
+
+    public void setCustomProperty(String property, String value) throws Exception {
+        if (propertySet.getPropertySetInfo().hasPropertyByName(property))
+            userProperties.removeProperty(property);
+        if (value != null)
+            userProperties.addProperty(property, com.sun.star.beans.PropertyAttribute.REMOVEABLE,
+                   new Any(Type.STRING, value));
+    }
+    
+    public String getCustomProperty(String property) throws Exception {
+        if (propertySet.getPropertySetInfo().hasPropertyByName(property))
+            return propertySet.getPropertyValue(property).toString();
+        else return null;
+
+    }
 
      public void updateSortedReferenceMarks() throws Exception {
          XReferenceMarksSupplier supplier = (XReferenceMarksSupplier) UnoRuntime.queryInterface(
@@ -213,9 +257,11 @@ public class OOBibBase {
      * @throws Exception
      */
     public void insertEntry(BibtexEntry[] entries, BibtexDatabase database, OOBibStyle style,
-                            boolean inParenthesis, boolean withText, boolean sync) throws Exception {
+                            boolean inParenthesis, boolean withText, String pageInfo,
+                            boolean sync) throws Exception {
 
         try {
+
             XTextViewCursor xViewCursor = xViewCursorSupplier.getViewCursor();
 
             if (entries.length > 1) {
@@ -238,11 +284,32 @@ public class OOBibBase {
                     withText ? (inParenthesis ? AUTHORYEAR_PAR : AUTHORYEAR_INTEXT) : INVISIBLE_CIT);
             //XTextContent content = insertBookMark(bName, xViewCursor);
 
+            // If we should store metadata for page info, do that now:
+            if (pageInfo != null) {
+                System.out.println("Storing page info: "+pageInfo);
+                setCustomProperty(bName, pageInfo);
+            }
+
 
             String citeText = style.isNumberEntries() ? "-" : style.getCitationMarker(entries, database, inParenthesis, null, null);
 
             //System.out.println(text+" / "+xViewCursor.getText());
             xViewCursor.getText().insertString(xViewCursor, " ", false);
+            if (style.isFormatCitations()) {
+                XPropertySet xCursorProps = (XPropertySet) UnoRuntime.queryInterface(
+                                    XPropertySet.class, xViewCursor);
+                String charStyle = style.getCitationCharacterFormat();
+                try {
+                    xCursorProps.setPropertyValue("CharStyleName", charStyle);
+                } catch (Throwable ex) {
+                    // Setting the character format failed, so we throw an exception that
+                    // will result in an error message for the user. Before that,
+                    // delete the space we inserted:
+                    xViewCursor.goLeft((short)1,true);
+                    xViewCursor.setString("");
+                    throw new UndefinedCharacterFormatException(charStyle);
+                }
+            }
             xViewCursor.goLeft((short)1,false);
             insertReferenceMark(bName, citeText, xViewCursor, withText, style);
             //xViewCursor.collapseToEnd();
@@ -281,11 +348,10 @@ public class OOBibBase {
      * @param database The database to get entries from.
      * @param style The bibliography style to use.
      * @return A list of those referenced BibTeX keys that could not be resolved.
-     * @throws UndefinedParagraphFormatException
      * @throws Exception
      */
     public List<String> refreshCiteMarkers(BibtexDatabase database, OOBibStyle style) throws
-            UndefinedParagraphFormatException, Exception {
+            Exception {
         try {
             return refreshCiteMarkersInternal(database, style);
         } catch (DisposedException ex) {
@@ -296,16 +362,32 @@ public class OOBibBase {
         }
     }
 
+    public XNameAccess getReferenceMarks() {
+        XReferenceMarksSupplier supplier = (XReferenceMarksSupplier) UnoRuntime.queryInterface(
+                XReferenceMarksSupplier.class, xCurrentComponent);
+        return supplier.getReferenceMarks();
+    }
+
+    public String[] getJabRefReferenceMarks(XNameAccess nameAccess) {
+        String[] names = nameAccess.getElementNames();
+        // Remove all reference marks that don't look like JabRef citations:
+        ArrayList<String> tmp = new ArrayList<String>();
+        for (int i = 0; i < names.length; i++) {
+            if (citePattern.matcher(names[i]).find())
+                tmp.add(names[i]);
+        }
+        names = tmp.toArray(new String[tmp.size()]);
+        return names;
+    }
+
     private List<String> refreshCiteMarkersInternal(BibtexDatabase database, OOBibStyle style) throws
-            UndefinedParagraphFormatException, Exception {
+            Exception {
 
         List<String> cited = findCitedKeys();
         List<BibtexEntry> entries = findCitedEntries(database, cited);
 
-        XReferenceMarksSupplier supplier = (XReferenceMarksSupplier) UnoRuntime.queryInterface(
-                XReferenceMarksSupplier.class, xCurrentComponent);
-        XNameAccess nameAccess = supplier.getReferenceMarks();
-        
+        XNameAccess nameAccess = getReferenceMarks();
+
         String[] names;
         if (style.isSortByPosition()) {
             // We need to sort the reference marks according to their order of appearance:
@@ -331,6 +413,14 @@ public class OOBibBase {
             names = sortedReferenceMarks;            
         }
 
+        // Remove all reference marks that don't look like JabRef citations:
+        ArrayList<String> tmp = new ArrayList<String>();
+        for (int i = 0; i < names.length; i++) {
+            if (citePattern.matcher(names[i]).find())
+                tmp.add(names[i]);
+        }
+        names = tmp.toArray(new String[tmp.size()]);
+
         HashMap<String,Integer> numbers = new HashMap<String, Integer>();
         //HashMap<S
         int lastNum = 0;
@@ -363,7 +453,18 @@ public class OOBibBase {
 
                 String[] normCitMarker = new String[keys.length];
                 String citationMarker;
-                if (style.isNumberEntries()) {
+                if (style.isBibtexKeyCiteMarkers()) {
+                    StringBuilder sb = new StringBuilder();
+                    normCitMarkers[i] = new String[keys.length];
+                    for (int j=0; j<keys.length; j++) {
+                        normCitMarkers[i][j] = cEntries[j].getCiteKey();
+                        sb.append(cEntries[j].getCiteKey());
+                        if (j < keys.length-1)
+                            sb.append(",");
+                    }
+                    citationMarker = sb.toString();
+                }
+                else if (style.isNumberEntries()) {
                     if (style.isSortByPosition()) {
                         // We have sorted the citation markers according to their order of appearance,
                         // so we simply count up for each marker referring to a new entry:
@@ -426,12 +527,14 @@ public class OOBibBase {
                 }
                 citMarkers[i] = citationMarker;
                 normCitMarkers[i] = normCitMarker;
+
             }
 
+
         }
 
         uniquefiers.clear();
-        if (!style.isNumberEntries()) {
+        if (!style.isBibtexKeyCiteMarkers() && !style.isNumberEntries()) {
             // See if there are duplicate citations marks referring to different entries. If so, we need to
             // use uniquefiers:
             HashMap<String,List<String>> refKeys = new HashMap<String, List<String>>();
@@ -513,13 +616,34 @@ public class OOBibBase {
 
         // Refresh all reference marks with the citation markers we computed:
         boolean hadBibSection = getBookmarkRange(BIB_SECTION_NAME) != null;
+        // Check if we are supposed to set a character format for citations:
+        boolean mustTestCharFormat = style.isFormatCitations();
         for (int i = 0; i < names.length; i++) {
             Object o = nameAccess.getByName(names[i]);
             XTextContent bm = (XTextContent) UnoRuntime.queryInterface
                     (XTextContent.class, o);
 
             XTextCursor cursor = bm.getAnchor().getText().createTextCursorByRange(bm.getAnchor());
+
+            if (mustTestCharFormat) {
+                // If we are supposed to set character format for citations, must run a test before we
+                // delete old citation markers. Otherwise, if the specified character format doesn't
+                // exist, we end up deleting the markers before the process crashes due to a the missing
+                // format, with catastrophic consequences for the user.
+                mustTestCharFormat = false; // need to do this only once
+                XPropertySet xCursorProps = (XPropertySet) UnoRuntime.queryInterface(
+                                    XPropertySet.class, cursor);
+                String charStyle = style.getCitationCharacterFormat();
+                try {
+                    xCursorProps.setPropertyValue("CharStyleName", charStyle);
+                } catch (Throwable ex) {
+                    throw new UndefinedCharacterFormatException(charStyle);
+                }
+            }
+
+
             text.removeTextContent(bm);
+
             insertReferenceMark(names[i], citMarkers[i], cursor, types[i] != INVISIBLE_CIT, style);
             if (hadBibSection && (getBookmarkRange(BIB_SECTION_NAME) == null)) {
                 // We have overwritten the marker for the start of the reference list.
@@ -661,6 +785,7 @@ public class OOBibBase {
         List<BibtexEntry> entries = findCitedEntries(database, cited);
         
         String[] names = sortedReferenceMarks;
+        
         if (style.isSortByPosition()) {
             // We need to sort the entries according to their order of appearance:
            entries = getSortedEntriesFromSortedRefMarks(names, database, entries);
@@ -668,8 +793,7 @@ public class OOBibBase {
         else {
             Collections.sort(entries, entryComparator);
         }
-        
-        clearBibTextSectionContent();
+        clearBibTextSectionContent2();
         populateBibTextSection(database, entries, style);
     }
 
@@ -804,6 +928,48 @@ public class OOBibBase {
             return null;
     }
 
+    public String getCitationContext(XNameAccess nameAccess, String refMarkName,
+                                     int charBefore, int charAfter,
+                                     boolean htmlMarkup) throws Exception {
+        Object o = nameAccess.getByName(refMarkName);
+        XTextContent bm = (XTextContent) UnoRuntime.queryInterface
+                (XTextContent.class, o);
+
+        XTextCursor cursor = bm.getAnchor().getText().createTextCursorByRange(bm.getAnchor());
+        String citPart = cursor.getString();
+        int flex = 8;
+        for (int i=0; i<charBefore; i++) {
+            try {
+                cursor.goLeft((short)1, true);
+                if ((i >= charBefore-flex) && Character.isWhitespace(cursor.getString().charAt(0)))
+                    break;
+            } catch (Exception ex) {
+                ex.printStackTrace();
+            }
+        }
+        int length = cursor.getString().length();
+        int added = length - citPart.length();
+        cursor.collapseToStart();
+        for (int i=0; i<charAfter+length; i++) {
+            try {
+                cursor.goRight((short)1, true);
+                if (i >= charAfter+length-flex) {
+                    String strNow = cursor.getString();
+                    if (Character.isWhitespace(strNow.charAt(strNow.length()-1)))
+                        break;
+                }
+            } catch (Exception ex) {
+                ex.printStackTrace();
+            }
+        }
+
+        String result = cursor.getString();
+        if (htmlMarkup) {
+            result = result.substring(0, added)+"<b>"+citPart+
+                    "</b>"+result.substring(length);
+        }
+        return result.trim();
+    }
 
     public void insertFullReferenceAtCursor(XTextCursor cursor, BibtexDatabase database,
                                             List<BibtexEntry> entries,
@@ -820,7 +986,7 @@ public class OOBibBase {
             if (style.isNumberEntries()) {
                 int minGroupingCount = style.getIntCitProperty("MinimumGroupingCount");
                 OOUtil.insertTextAtCurrentLocation(text, cursor,
-                        style.getNumCitationMarker(new int[] {number++}, minGroupingCount, true)+" ",
+                        style.getNumCitationMarker(new int[] {number++}, minGroupingCount, true),
                         false, false, false, false, false, false);
             }
             Layout layout = style.getReferenceFormat(entry.getType().getName());
@@ -873,14 +1039,53 @@ public class OOBibBase {
         return false;
     }
 
+    public void createBibTextSection2(boolean end) throws Exception {
+
+        XTextCursor mxDocCursor = text.createTextCursor();
+        if (end)
+            mxDocCursor.gotoEnd(false);
+        OOUtil.insertParagraphBreak(text, mxDocCursor);
+        // Create a new TextSection from the document factory and access it's XNamed interface
+        XNamed xChildNamed = UnoRuntime.queryInterface(
+          XNamed.class, mxDocFactory.createInstance("com.sun.star.text.TextSection"));
+        // Set the new sections name to 'Child_Section'
+        xChildNamed.setName(BIB_SECTION_NAME);
+        // Access the Child_Section's XTextContent interface and insert it into the document
+         XTextContent xChildSection = (XTextContent) UnoRuntime.queryInterface(
+             XTextContent.class, xChildNamed);
+         text.insertTextContent (mxDocCursor, xChildSection, false);
+        
+    }
+
+    public void clearBibTextSectionContent2() throws Exception {
+
+        // Check if the section exists:
+        boolean exists = false;
+        XTextSectionsSupplier supp = UnoRuntime.queryInterface(
+                XTextSectionsSupplier.class, mxDoc);
+        try {
+            XTextSection section = (XTextSection)((Any)supp.getTextSections().getByName(BIB_SECTION_NAME)).getObject();
+            // Clear it:
+            XTextCursor cursor = text.createTextCursorByRange(section.getAnchor());
+            cursor.gotoRange(section.getAnchor(), false);
+            cursor.setString("");
+
+        } catch (NoSuchElementException ex) {
+            createBibTextSection2(atEnd);
+        }
+    }
+
     public void clearBibTextSectionContent() throws Exception {
         // Get a range comparator:
         XTextRangeCompare compare = (XTextRangeCompare) UnoRuntime.queryInterface
                 (XTextRangeCompare.class, text);
         // Find the bookmarks for the bibliography:
         XTextRange range = getBookmarkRange(BIB_SECTION_NAME);
-        if (range == null)
+        if (range == null) {
             createBibTextSection(atEnd);
+        }
+
+
         XTextRange rangeEnd = getBookmarkRange(BIB_SECTION_END_NAME);
         if (rangeEnd == null) {
             // No end bookmark. This means that there is no bibliography.
@@ -906,8 +1111,10 @@ public class OOBibBase {
     public void populateBibTextSection(BibtexDatabase database, List<BibtexEntry> entries,
                                        OOBibStyle style)
             throws UndefinedParagraphFormatException, Exception {
-        XTextRange range = getBookmarkRange(BIB_SECTION_NAME);
-        XTextCursor cursor = text.createTextCursorByRange(range.getEnd());
+        XTextSectionsSupplier supp = UnoRuntime.queryInterface(XTextSectionsSupplier.class, mxDoc);
+        XTextSection section = (XTextSection)((Any)supp.getTextSections().getByName(BIB_SECTION_NAME))
+                    .getObject();
+        XTextCursor cursor = text.createTextCursorByRange(section.getAnchor());
         OOUtil.insertTextAtCurrentLocation(text, cursor, (String)style.getProperty("Title"),
                 (String)style.getProperty("ReferenceHeaderParagraphFormat"));
         insertFullReferenceAtCursor(cursor, database, entries, style,
@@ -934,6 +1141,14 @@ public class OOBibBase {
     public void insertReferenceMark(String name, String citText, XTextCursor position, boolean withText,
                                OOBibStyle style)
             throws Exception {
+
+        // Check if there is "page info" stored for this citation. If so, insert it into 
+        // the citation text before inserting the citation:
+        String pageInfo = getCustomProperty(name);
+        if (pageInfo != null) {
+            citText = style.insertPageInfo(citText, pageInfo);
+        }
+
         Object bookmark = mxDocFactory.createInstance("com.sun.star.text.ReferenceMark");
         // Name the reference
         XNamed xNamed = (XNamed) UnoRuntime.queryInterface(
@@ -948,11 +1163,19 @@ public class OOBibBase {
                 position.setString(citText);
                 XPropertySet xCursorProps = (XPropertySet) UnoRuntime.queryInterface(
                     XPropertySet.class, position);
+
                 // Set language to [None]:
                 xCursorProps.setPropertyValue("CharLocale", new Locale("zxx", "", ""));
-
-                // See if we should format the citation marker or not:
                 if (style.isFormatCitations()) {
+                    String charStyle = style.getCitationCharacterFormat();
+                    try {
+                        xCursorProps.setPropertyValue("CharStyleName", charStyle);
+                    } catch (Throwable ex) {
+                        throw new UndefinedCharacterFormatException(charStyle);
+                    }
+                }
+                // See if we should format the citation marker or not:
+                /*if (style.isFormatCitations()) {
 
                     if (style.getBooleanCitProperty("SuperscriptCitations")) {
                         xCursorProps.setPropertyValue("CharEscapement",
@@ -980,7 +1203,7 @@ public class OOBibBase {
                             style.isBoldCitations() ? com.sun.star.awt.FontWeight.BOLD :
                                 com.sun.star.awt.FontWeight.NORMAL);
                     
-                }
+                } */
             }
             else
                 position.setString("");
@@ -1107,6 +1330,23 @@ public class OOBibBase {
             String text = mxDocCursor.getString();
             // Check if the string contains no line breaks and only whitespace:
             if ((text.indexOf('\n') == -1) && (text.trim().length() == 0)) {
+
+                // If we are supposed to set character format for citations, test this before
+                // making any changes. This way we can throw an exception before any reference
+                // marks are removed, preventing damage to the user's document:
+                if (style.isFormatCitations()) {
+                    XPropertySet xCursorProps = (XPropertySet) UnoRuntime.queryInterface(
+                                        XPropertySet.class, mxDocCursor);
+                    String charStyle = style.getCitationCharacterFormat();
+                    try {
+                        xCursorProps.setPropertyValue("CharStyleName", charStyle);
+                    } catch (Throwable ex) {
+                        // Setting the character format failed, so we throw an exception that
+                        // will result in an error message for the user:
+                        throw new UndefinedCharacterFormatException(charStyle);
+                    }
+                }
+
                 List<String> keys = parseRefMarkName(names[piv]);
                 keys.addAll(parseRefMarkName(names[piv+1]));
                 removeReferenceMark(names[piv]);
