@@ -34,11 +34,11 @@ public class OOBibStyle implements Comparable {
     String name = null;
     SortedSet<String> journals = new TreeSet<String>();
 
+    // Formatter to be run on fields before they are used as part of citation marker:
+    LayoutFormatter fieldFormatter = new OOPreFormatter();
+
     Layout defaultBibLayout;
-    // formatters mapped from field names. Each field can have no mapping, or a set of formatters:
-    HashMap formatters = new HashMap();
-    // 0-n formatters applied to all fields before field-speficic formatters:
-    LayoutFormatter[] allBeforeFormat, allAfterFormat;
+
     // reference layout mapped from entry type number:
     HashMap<String, Layout> bibLayout = new HashMap<String, Layout>();
 
@@ -62,6 +62,9 @@ public class OOBibStyle implements Comparable {
     private static long styleFileModificationTime = Long.MIN_VALUE;
     private String COMBINED_ENTRIES_SEPARATOR = "-";
 
+    //private Pattern quoted = Pattern.compile("\".*^\\\\\"");
+    private Pattern quoted = Pattern.compile("\".*\"");
+
     public OOBibStyle(File styleFile) throws Exception {
         this(new FileReader(styleFile));
         this.styleFile = styleFile;
@@ -94,14 +97,17 @@ public class OOBibStyle implements Comparable {
         citProperties.put("BracketBefore", "(");
         citProperties.put("BracketAfter", ")");
         citProperties.put("CitationSeparator", "; ");
+        citProperties.put("PageInfoSeparator", "; ");
         citProperties.put("GroupedNumbersSeparator", "-");
         citProperties.put("MinimumGroupingCount", 3);
         citProperties.put("FormatCitations", Boolean.FALSE);
+        citProperties.put("CitationCharacterFormat", "Default");
         citProperties.put("ItalicCitations", Boolean.FALSE);
         citProperties.put("BoldCitations", Boolean.FALSE);
         citProperties.put("SuperscriptCitations", Boolean.FALSE);
         citProperties.put("SubscriptCitations", Boolean.FALSE);
         citProperties.put("MultiCiteChronological", Boolean.TRUE);
+        citProperties.put("BibtexKeyCitations", Boolean.FALSE);
 
         initialize(in);
 
@@ -123,7 +129,6 @@ public class OOBibStyle implements Comparable {
     private void initialize(Reader in) throws IOException {
         name = null;
         readFormatFile(in);
-        allBeforeFormat = new LayoutFormatter[] {new RemoveLatexCommands()};
     }
 
     /**
@@ -174,6 +179,8 @@ public class OOBibStyle implements Comparable {
 
         for (int i = 0; i < lines.length; i++) {
             String line = lines[i];
+            if ((line.length() > 0) && (line.charAt(line.length()-1) == '\r'))
+                line = line.substring(0, line.length()-1);
             // Check for empty line or comment:
             if ((line.trim().length() == 0) || (line.charAt(0) == '#'))
                 continue;
@@ -277,6 +284,8 @@ public class OOBibStyle implements Comparable {
         if ((index > 0) && (index <= line.length()-1)) {
             String propertyName = line.substring(0, index).trim();
             String value = line.substring(index+1);
+            if ((value.trim().length() > 2) && quoted.matcher(value.trim()).matches())
+                value = value.trim().substring(1, value.trim().length()-1);
             Object toSet = value;
             if (numPattern.matcher(value).matches()) {
                 toSet = Integer.parseInt(value);
@@ -320,36 +329,6 @@ public class OOBibStyle implements Comparable {
             return defaultBibLayout;
     }*/
 
-    /**
-     * Format the given field value based on the rules of this bib style.
-     * @param field The name of the field.
-     * @param content The unformatted field content.
-     * @return The formatted field value.
-     */
-    public String formatField(String field, String content) {
-        if (allBeforeFormat != null)
-            for (int i = 0; i < allBeforeFormat.length; i++) {
-                LayoutFormatter formatter = allBeforeFormat[i];
-                content = formatter.format(content);
-            }
-
-        Object o = formatters.get(field);
-        if (o == null)
-            return content;
-        LayoutFormatter[] form = (LayoutFormatter[])o;
-        for (int i = 0; i < form.length; i++) {
-            LayoutFormatter formatter = form[i];
-            content = formatter.format(content);
-        }
-
-        if (allAfterFormat != null)
-            for (int i = 0; i < allAfterFormat.length; i++) {
-                LayoutFormatter formatter = allAfterFormat[i];
-                content = formatter.format(content);
-            }
-
-        return content;
-    }
 
     /**
      * Format a number-based citation marker for the given number.
@@ -730,8 +709,11 @@ public class OOBibStyle implements Comparable {
         for (int i = 0; i < fields.length; i++) {
             String s = fields[i];
             String content = BibtexDatabase.getResolvedField(s, entry, database);
-            if ((content != null) && (content.trim().length() > 0))
+            if ((content != null) && (content.trim().length() > 0)) {
+                if (fieldFormatter != null)
+                    content = fieldFormatter.format(content);
                 return content;
+            }
         }
         // No luck? Return an empty string:
         return "";
@@ -762,6 +744,22 @@ public class OOBibStyle implements Comparable {
         return sb.toString();
     }
 
+    /**
+     * Take a finished citation and insert a string at the end (but inside the end bracket)
+     * separated by "PageInfoSeparator"
+     * @param citation
+     * @param pageInfo
+     * @return
+     */
+    public String insertPageInfo(String citation, String pageInfo) {
+        String bracketAfter = getStringCitProperty("BracketAfter");
+        if (citation.endsWith(bracketAfter)) {
+            String first = citation.substring(0, citation.length()-bracketAfter.length());
+            return first+getStringCitProperty("PageInfoSeparator")+pageInfo+bracketAfter;
+        }
+        else return citation+getStringCitProperty("PageInfoSeparator")+pageInfo;
+    }
+
     /**
      * Convenience method for checking the property for whether we use number citations or
      * author-year citations.
@@ -808,6 +806,10 @@ public class OOBibStyle implements Comparable {
         return (Boolean)citProperties.get("FormatCitations");
     }
 
+    public boolean isBibtexKeyCiteMarkers() {
+        return (Boolean)citProperties.get("BibtexKeyCitations");
+    }
+    
     /**
      * Get boolean property.
      * @param key The property key
@@ -820,6 +822,14 @@ public class OOBibStyle implements Comparable {
     public int getIntCitProperty(String key) {
         return (Integer)citProperties.get(key);
     }
+
+    public String getStringCitProperty(String key) {
+        return (String)citProperties.get(key);
+    }
+
+    public String getCitationCharacterFormat() {
+        return (String)citProperties.get("CitationCharacterFormat");
+    }
     /**
      * Get a style property.
      * @param name The property name.
@@ -837,4 +847,6 @@ public class OOBibStyle implements Comparable {
     public boolean equals(Object o) {
         return styleFile.equals(((OOBibStyle)o).styleFile);
     }
+
+
 }
