@@ -58,6 +58,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         pushEntries = new JButton(Globals.lang("Cite")),
         pushEntriesInt = new JButton(Globals.lang("Cite in-text")),
         pushEntriesEmpty = new JButton(Globals.lang("Insert empty citation")),
+        pushEntriesAdvanced = new JButton(Globals.lang("Cite special")),
         focus = new JButton("Focus OO document"),
         update,
         insertFullRef = new JButton("Insert reference text"),
@@ -111,9 +112,11 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         Globals.prefs.putDefaultValue("ooInParCitation", true);
         Globals.prefs.putDefaultValue("syncOOWhenCiting", false);
 
+        Globals.prefs.putDefaultValue("showOOPanel", true);
         
         styleFile = Globals.prefs.get("ooBibliographyStyleFile");
 
+
     }
 
     public SidePaneComponent getSidePaneComponent() {
@@ -133,6 +136,8 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
     }
 
     public JMenuItem getMenuItem() {
+        if (Globals.prefs.getBoolean("showOOPanel"))
+            manager.show(getName());
         JMenuItem item = new JMenuItem("OpenOffice.org panel", GUIGlobals.getImage("openoffice"));
         item.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent event) {
@@ -149,11 +154,13 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
 
     private void initPanel() throws Exception {
 
-        connect.addActionListener(new ActionListener() {
+        Action al = new AbstractAction() {
             public void actionPerformed(ActionEvent e) {
                 connect(true);
             }
-        });
+        };
+        connect.addActionListener(al);
+
         manualConnect.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent event) {
                 connect(false);
@@ -191,19 +198,25 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         pushEntries.setToolTipText(Globals.lang("Cite selected entries"));
         pushEntries.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                pushEntries(true, true);
+                pushEntries(true, true, false);
             }
         });
         pushEntries.setToolTipText(Globals.lang("Cite selected entries with in-text citation"));
         pushEntriesInt.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                pushEntries(false, true);
+                pushEntries(false, true, false);
             }
         });
         pushEntriesEmpty.setToolTipText(Globals.lang("Insert a citation without text (the entry will appear in the reference list)"));
         pushEntriesEmpty.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent event) {
-                pushEntries(false, false);
+                pushEntries(false, false, false);
+            }
+        });
+        pushEntriesAdvanced.setToolTipText(Globals.lang("Cite selected entries with extra information"));
+        pushEntriesAdvanced.addActionListener(new ActionListener() {
+            public void actionPerformed(ActionEvent event) {
+                pushEntries(false, true, true);
             }
         });
 
@@ -213,7 +226,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             }
         });
         update.setToolTipText(Globals.lang("Ensure that the bibliography is up-to-date"));
-        update.addActionListener(new ActionListener() {
+        Action updateAction = new AbstractAction() {
             public void actionPerformed(ActionEvent e) {
                 try {
                     style.ensureUpToDate();
@@ -226,6 +239,8 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
                         JOptionPane.showMessageDialog(frame, Globals.lang("Your OpenOffice document references the BibTeX key '%0', which could not be found in your current database.",
                             unresolvedKeys.get(0)), Globals.lang("Unable to synchronize bibliography"), JOptionPane.ERROR_MESSAGE);
                     }
+                } catch (UndefinedCharacterFormatException ex) {
+                    reportUndefinedCharacterFormat(ex);
                 } catch (UndefinedParagraphFormatException ex) {
                     reportUndefinedParagraphFormat(ex);
                 } catch (ConnectionLostException ex) {
@@ -238,7 +253,8 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
                     e1.printStackTrace();
                 }
             }
-        });
+        };
+        update.addActionListener(updateAction);
 
         insertFullRef.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent event) {
@@ -255,6 +271,8 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             public void actionPerformed(ActionEvent event) {
                 try {
                     ooBase.combineCiteMarkers(frame.basePanel().database(), style);
+                } catch (UndefinedCharacterFormatException e) {
+                    reportUndefinedCharacterFormat(e);
                 } catch (Exception e) {
                     e.printStackTrace();
                 }
@@ -271,7 +289,12 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         test.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent event) {
                 try {
-                    ooBase.testFrameHandling();
+                    CitationManager cm = new CitationManager(frame, ooBase);
+                    cm.showDialog();
+                    //pushEntries(false, true, true);
+
+                    //ooBase.testFrameHandling();
+
                     //ooBase.combineCiteMarkers(frame.basePanel().database(), style);
                     //insertUsingBST();
                     //ooBase.testFootnote();
@@ -289,6 +312,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         pushEntries.setEnabled(false);
         pushEntriesInt.setEnabled(false);
         pushEntriesEmpty.setEnabled(false);
+        pushEntriesAdvanced.setEnabled(false);
         focus.setEnabled(false);
         update.setEnabled(false);
         insertFullRef.setEnabled(false);
@@ -316,6 +340,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         b.append(setStyleFile);
         b.append(pushEntries);
         b.append(pushEntriesInt);
+        b.append(pushEntriesAdvanced);
         b.append(pushEntriesEmpty);
         b.append(merge);
         b.append(settingsB);
@@ -330,7 +355,12 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         comp.setContent(content);
         content.setLayout(new BorderLayout());
         content.add(b.getPanel(), BorderLayout.CENTER);
-        
+
+        System.out.println("Adding shortcuts");
+        frame.getTabbedPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
+                .put(Globals.prefs.getKey("Refresh OO"), "Refresh OO");
+        frame.getTabbedPane().getActionMap().put("Refresh OO", updateAction);
+
         //diag.pack();
         //diag.setVisible(true);
     }
@@ -384,46 +414,31 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             String ooJars = Globals.prefs.get("ooJarsPath");
             sOffice = Globals.prefs.get("ooExecutablePath");
 
-            boolean openOffice3 = Globals.prefs.getBoolean("connectToOO3");
+            boolean openOffice3 = true;//Globals.prefs.getBoolean("connectToOO3");
             if (Globals.ON_WIN) {
-                if (openOffice3) {
+                //if (openOffice3) {
                     unoilDir = ooPath+"\\Basis\\program\\classes";
                     ooBaseDirectory = ooPath+"\\URE\\java";
                     sOffice = ooPath+"\\program\\soffice.exe";
-                }
-                else {
-                    unoilDir = ooPath+"\\program\\classes";
-                    ooBaseDirectory = unoilDir;
-                    sOffice = ooPath+"\\program\\soffice.exe";
-                }
-
+                //}
+                
             }
             else if (Globals.ON_MAC) {
-                if (openOffice3) {
+                //if (openOffice3) {
                     sOffice = ooPath+"/Contents/MacOS/soffice.bin";
                     ooBaseDirectory = ooPath+"/Contents/basis-link/ure-link/share/java";
                     unoilDir = ooPath+"/Contents/basis-link/program/classes"; 
-                }
-                else {
-                    sOffice = ooPath+"/Contents/MacOS/soffice.bin";
-                    ooBaseDirectory = ooPath+"/Contents/MacOS/classes";
-                    unoilDir = ooPath+"/Contents/MacOS/classes";
-                }
+                //}
+
             }
             else {
                 // Linux:
-                if (openOffice3) {
+                //if (openOffice3) {
                     unoilDir = ooJars+"/program/classes";
                     ooBaseDirectory = ooJars+"/ure-link/share/java";
                     //sOffice = ooPath+"/program/soffice";
-                }
-                else {
-                    unoilDir = ooJars;
-                    ooBaseDirectory = ooJars;
-                    System.out.println(sOffice);
-                    System.out.println(ooJars);
-                    //sOffice = ooPath+"/program/soffice";
-                }
+                //}
+
             }
         }
 
@@ -434,19 +449,18 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
 
         // Add OO jars to the classpath:
         try {
-
-            URL[] jarList = new URL[] {
-                new File(unoilDir, "unoil.jar").toURI().toURL(),
-                new File(ooBaseDirectory, "jurt.jar").toURI().toURL(),
-                new File(ooBaseDirectory, "juh.jar").toURI().toURL(),
-                new File(ooBaseDirectory, "ridl.jar").toURI().toURL(),
-            };
+            File[] jarFiles = new File[] {
+                    new File(unoilDir, "unoil.jar"),
+                    new File(ooBaseDirectory, "jurt.jar"),
+                    new File(ooBaseDirectory, "juh.jar"),
+                    new File(ooBaseDirectory, "ridl.jar")};
+            URL[] jarList = new URL[jarFiles.length];
+            for (int i = 0; i < jarList.length; i++) {
+                if (!jarFiles[i].exists())
+                    throw new Exception(Globals.lang("File not found")+": "+jarFiles[i].getPath());
+                jarList[i] = jarFiles[i].toURI().toURL();
+            }
             addURL(jarList);
-            /*ClassLoader aCL = Thread.currentThread().getContextClassLoader();
-            loader = new URLClassLoader(jarList, aCL);
-            Thread.currentThread().setContextClassLoader(aUrlCL);
-            loader = new URLClassLoader(jarList);
-            */
 
             if (styleFile == null) {
                 JOptionPane.showMessageDialog(diag, "You must choose a style file before you can connect.", "No style file selected", JOptionPane.ERROR_MESSAGE);
@@ -474,6 +488,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             pushEntries.setEnabled(true);
             pushEntriesInt.setEnabled(true);
             pushEntriesEmpty.setEnabled(true);
+            pushEntriesAdvanced.setEnabled(true);
             focus.setEnabled(true);
             update.setEnabled(true);
             insertFullRef.setEnabled(true);
@@ -482,10 +497,11 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
 
         } catch (Exception e) {
             e.printStackTrace();
-            
+
             JOptionPane.showMessageDialog(frame, Globals.lang("Could not connect to running OpenOffice.\n"
-                +"Make sure you have the UNO jars (unoil.jar, juh.jar, jurt.jar and ridl.jar) on your classpath,\n"
-                +"as well as the OpenOffice bin directory.\nError message: "+e.getMessage()));
+                +"Make sure you have installed OpenOffice with Java support.\nIf connecting manually, please verify program and library paths.\n"
+                +"\nError message: "+e.getMessage()));
+
         }
     }
 
@@ -567,11 +583,6 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         browseOOJars.addActionListener(new BrowseAction(null, ooJars, true));
         ooExec.setText(Globals.prefs.get("ooExecutablePath"));
         ooJars.setText(Globals.prefs.get("ooJarsPath"));
-        final JRadioButton oo2 = new JRadioButton("OpenOffice.org 2.x", !Globals.prefs.getBoolean("connectToOO3"));
-        final JRadioButton oo3 = new JRadioButton("OpenOffice.org 3.x", Globals.prefs.getBoolean("connectToOO3"));
-        ButtonGroup bg = new ButtonGroup();
-        bg.add(oo2);
-        bg.add(oo3);
         DefaultFormBuilder builder = new DefaultFormBuilder(new FormLayout("left:pref, 4dlu, fill:pref:grow, 4dlu, fill:pref", ""));
         if (Globals.ON_WIN || Globals.ON_MAC) {
             builder.append(Globals.lang("Path to OpenOffice directory"));
@@ -590,10 +601,6 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             builder.append(browseOOJars);
             builder.nextLine();
         }
-        builder.append(Globals.lang("Program version"));
-        builder.append(oo2); builder.nextLine();
-        builder.append(new JPanel());
-        builder.append(oo3); builder.nextLine();
 
         ButtonBarBuilder bb = new ButtonBarBuilder();
         JButton ok = new JButton(Globals.lang("Ok"));
@@ -602,7 +609,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         ActionListener tfListener = new ActionListener() {
             public void actionPerformed(ActionEvent event) {
                 updateConnectionParams(ooPath.getText(), ooExec.getText(), ooJars.getText(),
-                        oo3.isSelected());
+                        true);
                 diag.dispose();
             }
         };
@@ -613,7 +620,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         ok.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent event) {
                 updateConnectionParams(ooPath.getText(), ooExec.getText(), ooJars.getText(),
-                        oo3.isSelected());
+                        true);
                 dialogOkPressed = true;
                 diag.dispose();
             }
@@ -640,13 +647,27 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
 
 
 
-    public void pushEntries(boolean inParenthesis, boolean withText) {
+    public void pushEntries(boolean inParenthesis, boolean withText, boolean addPageInfo) {
         if (!ooBase.isConnectedToDocument()) {
             JOptionPane.showMessageDialog(frame, Globals.lang("Not connected to any Writer document. Please"
                 +" make sure a document is open, and use the 'Select Writer document' button to connect to it."),
                     Globals.lang("Error"), JOptionPane.ERROR_MESSAGE);
             return;
         }
+
+        String pageInfo = null;
+        if (addPageInfo) {
+            AdvancedCiteDialog acd = new AdvancedCiteDialog(frame);
+            acd.showDialog();
+            if (acd.cancelled())
+                return;
+            if (acd.getPageInfo().length() > 0)
+                pageInfo = acd.getPageInfo();
+            inParenthesis = acd.isInParenthesisCite();
+
+        }
+
+
         BasePanel panel =frame.basePanel();
         final BibtexDatabase database = panel.database();
         if (panel != null) {
@@ -654,9 +675,11 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             if (entries.length > 0) {
                 try {
                     ooBase.insertEntry(entries, database, style, inParenthesis, withText,
-                            Globals.prefs.getBoolean("syncOOWhenCiting"));
+                            pageInfo, Globals.prefs.getBoolean("syncOOWhenCiting"));
                 } catch (ConnectionLostException ex) {
                     showConnectionLostErrorMessage();
+                } catch (UndefinedCharacterFormatException ex) {
+                    reportUndefinedCharacterFormat(ex);
                 } catch (UndefinedParagraphFormatException ex) {
                    reportUndefinedParagraphFormat(ex);
                 } catch (Exception ex) {
@@ -708,6 +731,13 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             Globals.lang(""), JOptionPane.ERROR_MESSAGE);
     }
 
+    private void reportUndefinedCharacterFormat(UndefinedCharacterFormatException ex) {
+        JOptionPane.showMessageDialog(frame, "<html>"+Globals.lang("Your style file specifies the character format '%0', "
+            +"which is undefined in your current OpenOffice document.", ex.getFormatName())+"<br>"
+            +Globals.lang("The character format is controlled by the citation property 'CitationCharacterFormat' in the style file.")
+            +"</html>",
+            Globals.lang(""), JOptionPane.ERROR_MESSAGE);
+    }
 
     public void insertUsingBST() {
         try {
@@ -753,11 +783,27 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
     public void pushEntries(boolean inParenthesis, BibtexEntry[] entries) {
         final BibtexDatabase database = frame.basePanel().database();
         if (entries.length > 0) {
+
+            String pageInfo = null;
+            //if (addPageInfo) {
+                AdvancedCiteDialog acd = new AdvancedCiteDialog(frame);
+                acd.showDialog();
+                if (acd.cancelled())
+                    return;
+                if (acd.getPageInfo().length() > 0)
+                    pageInfo = acd.getPageInfo();
+                inParenthesis = acd.isInParenthesisCite();
+
+            //}
+
+
             try {
                 ooBase.insertEntry(entries, database, style, inParenthesis, true,
-                    Globals.prefs.getBoolean("syncOOWhenCiting"));
+                    pageInfo, Globals.prefs.getBoolean("syncOOWhenCiting"));
             } catch (ConnectionLostException ex) {
                 showConnectionLostErrorMessage();
+            } catch (UndefinedCharacterFormatException ex) {
+                reportUndefinedCharacterFormat(ex);
             } catch (UndefinedParagraphFormatException ex) {
                reportUndefinedParagraphFormat(ex);
             } catch (Exception ex) {
@@ -787,9 +833,10 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
     }
 
     public JPanel getSettingsPanel() {
-        if (settings == null)
+        return null;
+        /*if (settings == null)
             initSettingsPanel();
-        return settings;
+        return settings;*/
     }
 
     private void initSettingsPanel() {
@@ -814,6 +861,7 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
             connect(true);
         }
         if (ooBase != null) {
+
             pushEntries(Globals.prefs.getBoolean("ooInParCitation"), entries);
         }
     }
@@ -835,6 +883,16 @@ public class OOTestPanel extends AbstractWorker implements SidePanePlugin, PushT
         public String getName() {
             return OOTestPanel.this.getName();
         }
+
+        @Override
+        public void componentClosing() {
+            Globals.prefs.putBoolean("showOOPanel", false);
+        }
+
+        @Override
+        public void componentOpening() {
+            Globals.prefs.putBoolean("showOOPanel", true);
+        }
     }
 
 
